﻿using System.Collections.Generic;
using System.Linq;
using MvcForum.DomainObjects;
using MvcForum.Infrastructure;
using MvcForum.Repository;
using MvcForum.ViewModels;

namespace MvcForum.Service
{
    public class ForumService : IForumService
    {
        private readonly IRepository<Category> categoryRepository;
        private readonly IRepository<Forum> forumRepository;
        private readonly IRepository<Thread> threadRepository;
        private readonly IRepository<Post> postRepository;
        private readonly IRepository<User> userRepository;
        private readonly IUnitOfWork unitOfWork;

        public ForumService(IRepository<Category> categoryRepository, IRepository<Forum> forumRepository, IRepository<Thread> threadRepository, IRepository<Post> postRepository, IRepository<User> userRepository, IUnitOfWork unitOfWork)
        {
            this.categoryRepository = categoryRepository;
            this.forumRepository = forumRepository;
            this.threadRepository = threadRepository;
            this.postRepository = postRepository;
            this.userRepository = userRepository;
            this.unitOfWork = unitOfWork;
        }

        public virtual CategoryViewModel GetCategoryById(int id)
        {
            Category category = categoryRepository.One(id);

            return CreateCategoryViewModel(category);
        }

        public virtual CategoryViewModel GetCategoryBySlug(string slug)
        {
            Category category = categoryRepository.FindOne(c => c.Slug == slug);

            return CreateCategoryViewModel(category);
        }

        public virtual IEnumerable<CategoryViewModel> GetCategories()
        {
            IEnumerable<CategoryViewModel> category = categoryRepository.All().Select(CreateCategoryViewModel);

            return category;
        }

        public virtual ForumViewModel GetForumById(int id)
        {
            Forum forum = forumRepository.One(id);

            return CreateForumViewModel(forum);
        }

        public virtual ForumViewModel GetForumByCategoryAndSlug(int categoryId, string slug)
        {
            Forum forum = forumRepository.FindOne(f => f.CategoryId == categoryId && f.Slug == slug);

            return CreateForumViewModel(forum);
        }

        public virtual IEnumerable<ForumViewModel> GetForumsByCategoryId(int id)
        {
            return forumRepository.FindAll(f => f.CategoryId == id)
                                  .Select(CreateForumViewModel);
        }

        public virtual ThreadViewModel GetThreadById(int id)
        {
            Thread thread = threadRepository.One(id);

            return CreateThreadViewModel(thread);
        }

        public virtual IEnumerable<ThreadViewModel> GetThreadsByForumId(int id)
        {
            return threadRepository.FindAll(t => t.Forum.Id == id)
                                   .Select(CreateThreadViewModel);
        }

        public virtual PostViewModel GetPostById(int id)
        {
            Post post = postRepository.One(id);

            return CreatePostViewModel(post);
        }

        public virtual IEnumerable<PostViewModel> GetPostsByThreadId(int id)
        {
            return postRepository.FindAll(p => p.Thread.Id == id)
                                 .Select(CreatePostViewModel);
        }

        public virtual int CreateThread(string title, string content, int forumId, int userId)
        {
            User user = userRepository.One(userId);

            Thread thread = new Thread
            {
                Title = title,
                Slug = title.ToUrlSafe(),
                Forum = forumRepository.One(forumId),
                User = user,
                CreatedAt = SystemTime.Now()
            };

            Post post = new Post
            {
                Thread = thread,
                Content = content ?? string.Empty,
                User = user,
                PostedAt = SystemTime.Now()
            };

            thread.Posts.Add(post);

            threadRepository.Add(thread);

            unitOfWork.Commit();

            return thread.Id;
        }

        public virtual int ReplyThread(string content, int threadId, int userId)
        {
            Thread thread = threadRepository.One(threadId);

            Post post = new Post
            {
                Thread = thread,
                Content = content ?? string.Empty,
                User = userRepository.One(userId),
                PostedAt = SystemTime.Now()
            };

            thread.Posts.Add(post);

            unitOfWork.Commit();

            return post.Id;
        }

        private static PostViewModel CreatePostViewModel(Post post)
        {
            return new PostViewModel
            {
                Id = post.Id,
                Content = post.Content ?? string.Empty,
                PostedAt = post.PostedAt,
                ThreadId = post.ThreadId,
                ThreadTitle = post.Thread.Title,
                ThreadSlug = post.Thread.Slug,
                UserName = post.User.Name,
                UserEmail = post.User.Email,
                IsPostedByOwner = post.IsPostedByOwner()
            };
        }

        private CategoryViewModel CreateCategoryViewModel(Category category)
        {
            return new CategoryViewModel
            {
                Id = category.Id,
                Name = category.Name,
                Slug = category.Slug,
                Order = category.Order,
                Forums = forumRepository.Count(f => f.Category.Id == category.Id)
            };
        }

        private ForumViewModel CreateForumViewModel(Forum forum)
        {
            return new ForumViewModel
            {
                Id = forum.Id,
                Name = forum.Name,
                Slug = forum.Slug,
                Order = forum.Order,
                CategoryId = forum.CategoryId,
                CategoryName = forum.Category.Name,
                CategorySlug = forum.Category.Slug,
                Threads = threadRepository.Count(t => t.Forum.Id == forum.Id)
            };
        }

        private ThreadViewModel CreateThreadViewModel(Thread thread)
        {
            return new ThreadViewModel
            {
                Id = thread.Id,
                Title = thread.Title,
                Slug = thread.Slug,
                CreatedAt = thread.CreatedAt,
                UserId = thread.UserId,
                UserName = thread.User.Name,
                UserEmail = thread.User.Email,
                ForumId = thread.ForumId,
                ForumName = thread.Forum.Name,
                ForumSlug = thread.Forum.Slug,
                Posts = postRepository.Count(p => p.Thread.Id == thread.Id)
            };
        }
    }
}