﻿using System.Web.Mvc;
using System.Web.Security;

namespace MvcForum.Web
{
    public class AccountController : Controller
    {
        private readonly IFormsAuthenticationService formsService;
        private readonly IMembershipService membershipService;

        public AccountController(IFormsAuthenticationService formsService, IMembershipService membershipService)
        {
            this.formsService = formsService;
            this.membershipService = membershipService;
        }

        public ActionResult Register()
        {
            return View(new RegistrationForm());
        }

        [HttpPost]
        public ActionResult Register(RegistrationForm form)
        {
            if (ModelState.IsValid)
            {
                // Attempt to register the user
                var createStatus = membershipService.CreateUser(form.UserName, form.Password, form.Email);

                if (createStatus == MembershipCreateStatus.Success)
                {
                    formsService.SignIn(form.UserName, false /* createPersistentCookie */);

                    return RedirectToAction("Index", "Home");
                }

                ModelState.AddModelError("", ErrorCodeToString(createStatus));
            }

            // If we got this far, something failed, redisplay form
            return View(form);
        }

        public ActionResult SignIn()
        {
            return View(new SignInForm());
        }

        [HttpPost]
        public ActionResult SignIn(SignInForm form, string returnUrl)
        {
            if (ModelState.IsValid)
            {
                if (membershipService.ValidateUser(form.UserName, form.Password))
                {
                    formsService.SignIn(form.UserName, form.RememberMe);

                    if (!returnUrl.IsNullOrWhiteSpace())
                    {
                        return Redirect(returnUrl);
                    }

                    return RedirectToAction("Index", "Home");
                }

                ModelState.AddModelError(string.Empty, "The user name or password provided is incorrect.");
            }

            return View(form);
        }

        [Authorize]
        public ActionResult ChangePassword()
        {
            return View(new ChangePasswordForm());
        }

        [Authorize, HttpPost]
        public ActionResult ChangePassword(ChangePasswordForm form)
        {
            if (ModelState.IsValid)
            {
                if (membershipService.ChangePassword(User.Identity.Name, form.OldPassword, form.NewPassword))
                {
                    return View("ChangePasswordSuccess");
                }

                ModelState.AddModelError(string.Empty, "The current password is incorrect or the new password is invalid.");
            }

            // If we got this far, something failed, redisplay form
            return View(form);
        }

        public ActionResult SignOut()
        {
            formsService.SignOut();

            return RedirectToAction("Index", "Home");
        }

        protected override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            ViewData["PasswordLength"] = membershipService.MinPasswordLength;

            base.OnActionExecuting(filterContext);
        }

        private static string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateUserName:
                    return Resources.Errors.DuplicateUserName;

                case MembershipCreateStatus.DuplicateEmail:
                    return Resources.Errors.DuplicateEmail;

                case MembershipCreateStatus.InvalidPassword:
                    return Resources.Errors.InvalidPassword;

                case MembershipCreateStatus.InvalidEmail:
                    return Resources.Errors.InvalidEmail;

                case MembershipCreateStatus.InvalidAnswer:
                    return Resources.Errors.InvalidAnswer;

                case MembershipCreateStatus.InvalidQuestion:
                    return Resources.Errors.InvalidQuestion;

                case MembershipCreateStatus.InvalidUserName:
                    return Resources.Errors.InvalidUserName;

                case MembershipCreateStatus.ProviderError:
                    return Resources.Errors.ProviderError;

                case MembershipCreateStatus.UserRejected:
                    return Resources.Errors.UserRejected;

                default:
                    return Resources.Errors.UserDefaultMessage;
            }
        }
    }
}